package sun.tools.jar;

import java.io.OutputStream;
import java.security.NoSuchAlgorithmException;
import java.util.Base64;
import java.io.PrintStream;
import java.io.ByteArrayOutputStream;
import sun.security.x509.AlgorithmId;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.Locale;
import java.security.MessageDigest;
import java.util.Hashtable;
import sun.security.pkcs.PKCS7;
import sun.net.www.MessageHeader;
import java.util.Vector;

public class SignatureFile
{
    static final boolean debug = false;
    private Vector<MessageHeader> entries;
    static final String[] hashes;
    private Manifest manifest;
    private String rawName;
    private PKCS7 signatureBlock;
    private Hashtable<String, MessageDigest> digests;
    
    static final void debug(String s) {
    }
    
    private SignatureFile(String s) throws JarException {
        this.entries = new Vector();
        this.digests = new Hashtable();
        this.entries = new Vector();
        if (s != null) {
            if (s.length() > 8 || s.indexOf(46) != -1) {
                throw new JarException("invalid file name");
            }
            this.rawName = s.toUpperCase(Locale.ENGLISH);
        }
    }
    
    private SignatureFile(String s, boolean b) throws JarException {
        this(s);
        if (b) {
            MessageHeader obj = new MessageHeader();
            obj.set("Signature-Version", "1.0");
            this.entries.addElement(obj);
        }
    }
    
    public SignatureFile(Manifest manifest, String s) throws JarException {
        this(s, true);
        this.manifest = manifest;
        Enumeration<MessageHeader> entries = manifest.entries();
        while (entries.hasMoreElements()) {
            String value = ((MessageHeader)entries.nextElement()).findValue("Name");
            if (value != null) {
                this.add(value);
            }
            else {
                continue;
            }
        }
    }
    
    public SignatureFile(Manifest manifest, String[] array, String s) throws JarException {
        this(s, true);
        this.manifest = manifest;
        this.add(array);
    }
    
    public SignatureFile(InputStream is, String s) throws IOException {
        this(s);
        while (is.available() > 0) {
            this.entries.addElement(new MessageHeader(is));
        }
    }
    
    public SignatureFile(InputStream inputStream) throws IOException {
        this(inputStream, null);
    }
    
    public SignatureFile(byte[] buf) throws IOException {
        this(new ByteArrayInputStream(buf));
    }
    
    public String getName() {
        return "META-INF/" + this.rawName + ".SF";
    }
    
    public String getBlockName() {
        String name = "DSA";
        if (this.signatureBlock != null) {
            name = this.signatureBlock.getSignerInfos()[0].getDigestEncryptionAlgorithmId().getName();
            String encAlgFromSigAlg = AlgorithmId.getEncAlgFromSigAlg(name);
            if (encAlgFromSigAlg != null) {
                name = encAlgFromSigAlg;
            }
        }
        return "META-INF/" + this.rawName + "." + name;
    }
    
    public PKCS7 getBlock() {
        return this.signatureBlock;
    }
    
    public void setBlock(PKCS7 signatureBlock) {
        this.signatureBlock = signatureBlock;
    }
    
    public void add(String[] array) throws JarException {
        for (int i = 0; i < array.length; ++i) {
            this.add(array[i]);
        }
    }
    
    public void add(String str) throws JarException {
        MessageHeader entry = this.manifest.getEntry(str);
        if (entry == null) {
            throw new JarException("entry " + str + " not in manifest");
        }
        MessageHeader computeEntry;
        try {
            computeEntry = this.computeEntry(entry);
        }
        catch (IOException ex) {
            throw new JarException(ex.getMessage());
        }
        this.entries.addElement(computeEntry);
        return;
    }
    
    public MessageHeader getEntry(String s) {
        Enumeration<MessageHeader> entries = this.entries();
        while (entries.hasMoreElements()) {
            MessageHeader messageHeader = (MessageHeader)entries.nextElement();
            if (s.equals(messageHeader.findValue("Name"))) {
                return messageHeader;
            }
            continue;
        }
        return null;
    }
    
    public MessageHeader entryAt(int index) {
        return (MessageHeader)this.entries.elementAt(index);
    }
    
    public Enumeration<MessageHeader> entries() {
        return this.entries.elements();
    }
    
    private MessageHeader computeEntry(MessageHeader messageHeader) throws IOException {
        MessageHeader messageHeader2 = new MessageHeader();
        String value = messageHeader.findValue("Name");
        if (value == null) {
            return null;
        }
        messageHeader2.set("Name", value);
        try {
            for (int i = 0; i < SignatureFile.hashes.length; ++i) {
                MessageDigest digest = this.getDigest(SignatureFile.hashes[i]);
                ByteArrayOutputStream out = new ByteArrayOutputStream();
                messageHeader.print(new PrintStream(out));
                messageHeader2.set(SignatureFile.hashes[i] + "-Digest", Base64.getMimeEncoder().encodeToString(digest.digest(out.toByteArray())));
            }
            return messageHeader2;
        }
        catch (NoSuchAlgorithmException ex) {
            throw new JarException(ex.getMessage());
        }
    }
    
    private MessageDigest getDigest(String key) throws NoSuchAlgorithmException {
        MessageDigest instance = (MessageDigest)this.digests.get(key);
        if (instance == null) {
            instance = MessageDigest.getInstance(key);
            this.digests.put(key, instance);
        }
        instance.reset();
        return instance;
    }
    
    public void stream(OutputStream out) throws IOException {
        MessageHeader messageHeader = (MessageHeader)this.entries.elementAt(0);
        if (messageHeader.findValue("Signature-Version") == null) {
            throw new JarException("Signature file requires Signature-Version: 1.0 in 1st header");
        }
        PrintStream printStream = new PrintStream(out);
        messageHeader.print(printStream);
        for (int i = 1; i < this.entries.size(); ++i) {
            ((MessageHeader)this.entries.elementAt(i)).print(printStream);
        }
        return;
    }
    
    static {
        hashes = new String[] { "SHA" };
    }
}
